/**
 * Title: Expert 13 - The JDBC paradigm: preparedStatement, TYPE_SCROLL and ISOLATION LEVELS
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.sql.*;

public class Expert13 {

	boolean test = false ;
	String url = "jdbc:odbc:test" ;

	// Running this application like "java Expert13 p" will populate the table PEOPLE.
	// Instead, using the options "t" will test the database features.
	public static void main(String args[]) {

		try {	Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");		}
		catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		Expert13 manager = new Expert13() ;
		if ( (args.length > 0) && (args[0].equalsIgnoreCase("p")) ) manager.populate() ;
		manager.test = (args.length > 0) && ( args[0].equalsIgnoreCase("t") ) ;
		manager.retrieve() ;
	}

	// Populating the table PEOPLE using the preparedStatement object
	public void populate() {

		int count = 0 ;
		Connection con = null ;
		PreparedStatement pstmt = null ;
		String query = "INSERT INTO people VALUES (?, ?, ?)" ;

		// REMARK: even if the objects "con" and "pstmt" are used more times in this class, it's better
		// to avoid defining them as class attributes, since they're not LOGICALLY bound to this instance.
		// (think about an application server which need to restore to the database)

		try {
			con = DriverManager.getConnection(url, "", "");
			pstmt = con.prepareStatement(query) ;

			for (int n = 1; n < 10; n++) {
				pstmt.setString(1, "Employee_" + n);	// Creating a NAME for the employee
				pstmt.setInt(2, n);						// Creating the PERSON_ID
				pstmt.setInt(3, n*2);					// Creating a fake AGE value
				count = pstmt.executeUpdate();
			}
			pstmt.close();
			con.close();
		}
		catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
			ex.printStackTrace() ;
		}
	}

	// Retrieving data from table PEOPLE
	public void retrieve() {

		int count = 0 ;
		Connection con = null ;
		PreparedStatement pstmt = null ;
		String query = "SELECT * FROM people WHERE person_id = ?" ;

		try {

			con = DriverManager.getConnection(url, "", "");
			// Try these diffent configurations to verify SQL exceptions:
			//pstmt = con.prepareStatement(query, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
			pstmt = con.prepareStatement(query, ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);

			// Remark: we specify the CONCUR_UPDATABLE value only because it's required by the signature of
			// the method: con.prepareStatement(x,x,x) (see JavaDocs).

			System.out.print("\nInsert row number: ") ;
			pstmt.setInt(1, getInput());
			ResultSet rs = pstmt.executeQuery();

			rs.last();	// if the ResultSet is not TYPE_FORWARD_ONLY, this method throws a SQLExpection
			System.out.println("Fetched: " + rs.getRow() + " rows\n");

			// Test the SENSITIVE feature: try to change data in the DB before pressing any key !!!
			System.out.print("Press 'Return' to display data ... ");
			count = System.in.read() ;

			rs.beforeFirst();		// necessary because display() reads data using rs.next() !!!
			Expert11.display(rs) ;
			if (test) test(rs) ;	// Analyze the ResultSet (see below)

			pstmt.close();
			con.commit() ;			// auto-commit is false because we have set transaction isolation !!!
			con.close();
		}
		catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
			ex.printStackTrace() ;
		}
		catch(Exception ex)    {	System.err.println("I/O Exception: " + ex.getMessage()); }
	}

	// Testing the features offered by the database.
	public void test(ResultSet rs) {

		Connection con = null ;

		try {
			rs.beforeFirst();
			con = DriverManager.getConnection(url, "", "");
			DatabaseMetaData database = con.getMetaData() ;
			System.out.println();
			System.out.println("Type Forward?          " + database.supportsResultSetType(ResultSet.TYPE_FORWARD_ONLY));
			System.out.println("Scroll Sensitive?      " + database.supportsResultSetType(ResultSet.TYPE_SCROLL_SENSITIVE));
			System.out.println("Scroll Insensitive?    " + database.supportsResultSetType(ResultSet.TYPE_SCROLL_INSENSITIVE));
			System.out.println("Forward + Updatable    " + database.supportsResultSetConcurrency(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE));
			System.out.println("Sensitive + Updatable  " + database.supportsResultSetConcurrency(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE));
			System.out.println("Insenstive + Updatable " + database.supportsResultSetConcurrency(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE));
			System.out.println("Forward + ReadOnly     " + database.supportsResultSetConcurrency(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY));
			System.out.println("Sensitive + ReadOnly   " + database.supportsResultSetConcurrency(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY));
			System.out.println("Insenstive + ReadOnly  " + database.supportsResultSetConcurrency(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY));
			System.out.println("\nCurrent Result Set (see JavaDocs: Costant Field Values)");
			System.out.println("Type:        " + rs.getType());
			System.out.println("Concurrency: " + rs.getConcurrency() + "\n");
			con.close();

		} catch (SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
			ex.printStackTrace() ;
		}
	}

// ******************************************	UTILITIES	*********************************************

	// Reading input data from the keyboard.
	// This method is defined static in order to use it in next exercises ...
	public static int getInput() {

		int n = 0 ;
		int output = 0;
		String input = "";

		try {
			while (n != 13) {
				n = System.in.read() ;
				if (n != 13) input += (char) n ;
			}
			// Skipping line feed '\n' (10), which follows the carriage return '\r' (13)
			System.in.skip(1) ;
			output = Integer.parseInt(input) ;
		}
		catch(Exception e) {
			System.err.println("Input Exception ... catched");
			return output ;		// return default value (zero)
		}

		return output ;
	}

} // End of class
